/* base64.cc - base64 implementation
 * Rob Earhart
 * $Id: base64.cc,v 1.1.1.1 1997/11/26 21:01:09 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <string.h>
#include "base64.h"

// Table from JGM's imapd lib
#define XX 127

static const unsigned char From64[256] = {
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,62, XX,XX,XX,63,
  52,53,54,55, 56,57,58,59, 60,61,XX,XX, XX, 0,XX,XX,
  XX, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
  15,16,17,18, 19,20,21,22, 23,24,25,XX, XX,XX,XX,XX,
  XX,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
  41,42,43,44, 45,46,47,48, 49,50,51,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX
};

pair<char *, unsigned> Base64::FromBase64(const char *str) {
  unsigned len = strlen(str);

  if (len % 4)
    throw IllegalBase64();

  len = (len / 4) * 3;

  char *result = new char[len];
  unsigned char *dest = (unsigned char *)result;
  const unsigned char *src = (const unsigned char *)str;

  try {
    while(src[0]) {
      if (From64[src[0]] == XX ||
	  From64[src[1]] == XX ||
	  From64[src[2]] == XX ||
	  From64[src[3]] == XX ||
	  src[2] == '=' && src[3] != '=' ||
	  src[3] == '=' && src[4] != '\0')
	throw IllegalBase64();
      *dest++ = From64[src[0]] << 2 | From64[src[1]] >> 4;
      *dest++ = From64[src[1]] << 4 | From64[src[2]] >> 2;
      *dest++ = From64[src[2]] << 6 | From64[src[3]];
      if (src[2] == '=') len--;
      if (src[3] == '=') len--;
      src += 4;
    }
  } catch (...) {
    delete [] result;
    throw;
  }
  return make_pair(result, len);
}

static const unsigned char To64[64] = {
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'P', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '/'
};

char *Base64::ToBase64(const char *buf, unsigned len) {
  // We allocate four bytes per three bytes of source,
  // rounding up, with one byte on the end for a terminator.
  char *result = new char[((len + 2) / 3) * 4 + 1];
  unsigned char *dest = (unsigned char *)result;
  const unsigned char *src = (const unsigned char *)buf;

  // This fills in groups of 3 bytes...
  while (len > 2) {
    dest[0] = To64[src[0] >> 2];
    dest[1] = To64[src[0] << 4 & 0x30 | src[1] >> 4];
    dest[2] = To64[src[1] << 2 & 0x3c | src[2] >> 6];
    dest[3] = To64[src[2] & 0x3f];
    dest += 4;
    src += 3;
    len -= 3;
  }

  // This fills in the trailing bit
  switch (len) {
  case 2:
    dest[0] = To64[src[0] >> 2];
    dest[1] = To64[src[0] << 4 & 0x30 | src[1] >> 4];
    dest[2] = To64[src[1] << 2 & 0x3c];
    dest[3] = '=';
    dest+=4;
    break;
  case 1:
    dest[0] = To64[src[0] >> 2];
    dest[1] = To64[src[0] << 4 & 0x30];
    dest[2] = '=';
    dest[3] = '=';
    dest+=4;
    break;
  }

  dest[0] = '\0';		// give it a terminator...
  return result;
}
