/* command.h -*- C++ -*- command interface
 * Rob Earhart
 * $Id: command.h,v 1.3 1998/01/08 23:40:18 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_COMMAND_H
#define ACAP_COMMAND_H

#include "ptr.h"
#include "str.h"
#include "task.h"
#include "refcount.h"
#include "conn.h"
#include "key.h"
#include "msg.h"

// A command is a task that's bound to a specific client request
class Command : public Task {
public:
  Command(const String &_tag, Conn *_conn)
    : tag(_tag), client(_conn) {}
  virtual ~Command() {}
  void Run() throw ();

  // send an alert to the client
  void ALERT(const char *msg);
  Conn *Client() const { return client; }
  const String &Tag() const { return tag; }

  static Conn *CurrentConn() {
    return conn_key.Get();
  }
protected:
  virtual void Eval() = 0;
  virtual Msg::Generator Name() const = 0;

private:
  String tag;
  P<Conn> client;
  static Key<Conn> conn_key;
};

#endif // ACAP_COMMAND_H
