/* cond.h -*- C++ -*- condition variable interface
 * Rob Earhart
 * $Id: cond.h,v 1.1 1997/12/19 22:25:10 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_COND_H
#define ACAP_COND_H

// This is just a wrapper around pthread_cond_t, providing automatic
// construction, destruction, syntactic sugar, and maybe a prayer of
// portability in the future (if we ever move to a non-pthreads
// system).

#include <pthread.h>

class Cond {
public:
  Cond() { pthread_cond_init(&cond, NULL); }
  ~Cond() { pthread_cond_destroy(&cond); }
  void Wait(pthread_mutex_t *mutex) { pthread_cond_wait(&cond, mutex); }
  void TimedWait(pthread_mutex_t *mutex, const struct timespec *abstime) {
    pthread_cond_timedwait(&cond, mutex, abstime);
  }
  void Signal() { pthread_cond_signal(&cond); }
  void Broadcast() { pthread_cond_broadcast(&cond); }
private:
  pthread_cond_t cond;
};

#endif // ACAP_COND_H
