/* context.h -*- C++ -*- context implementation
 * Rob Earhart
 * $Id: context.h,v 1.9 1998/03/09 00:58:02 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_CONTEXT_H
#define ACAP_CONTEXT_H

#include <set.h>
#include "response.h"
#include "refcount.h"
#include "str.h"
#include "timestamp.h"
#include "dispatch.h"
#include "searchable.h"

class Conn;
class NotifyContext;

class Context : public Searchable {
public:
  Context(Conn *conn,
	  const Timestamp::Stamp &modtime,
	  bool enumerated);
  virtual ~Context();

  // called after context initialization
  virtual void DoneInitializing(Context::entries_t &ents,
				Ptr<DatasetGroup> &group,
				Ptr<Criteria> &crit,
				Ptr<Modifiers> &mods);

  // I *really* wish Solaris's CC would get with the times and implement
  // dynamic_cast<>(); this is a horrid hack...
  virtual NotifyContext *NotifyCast() { return NULL; }
  
  typedef enum { bad, good } state_t;
  state_t State() const { return state; }
  Mutex &StateMutex() { return state_mutex; }

  void Search(entries_t &entries,
	      DatasetGroup *group,
	      NotifyContext *context,
	      const String &path,
	      Modifiers *modifiers,
	      Criteria *criteria,
	      Referer *referer,
	      Timestamp::Stamp stamp);

  bool Enumerated() const { return enumerated; }

  static void CheckLegalContextName(const String &path) {
    if (find(path.begin(), path.end(), '/') != path.end())
      throw Response::BAD("Context paths shouldn't contain '/'s");
  }
  static bool IsDatasetPath(const String &path) {
    return ! path.size() || path[0] == '/';
  }
protected:
  Ptr<DatasetGroup> dsgroup;
  state_t state;
  Conn *Client() const { return conn; }
  entries_t entries;
  Timestamp::Stamp modtime;

  virtual void Enumerate(entries_t &result,
			 DatasetGroup *group,
			 Timestamp::Stamp stamp) = 0;

private:
  Conn *conn;
  Mutex state_mutex;
  bool enumerated;
};

#endif // ACAP_CONTEXT_H
