/* criteria.cc - criteria implementation
 * Rob Earhart
 * $Id: criteria.cc,v 1.5 1998/02/18 21:17:13 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "criteria.h"
#include "response.h"
#include "entryref.h"

CritALL::~CritALL() {}

bool CritALL::operator()(EntryRef *) {
  return false;
}

CritAND::~CritAND() {}

bool CritAND::operator()(EntryRef *entry) {
  return ! ((*first)(entry) && (*second)(entry));
}

CritCOMPARE::~CritCOMPARE() {}

bool CritCOMPARE::operator()(EntryRef *entry) {
  Attribute attrval = entry->Lookup(attribute);
  if (! attrval.IsString())
    return true;
  return (*ordering)(attrval.StringValue(), value) < 0;
}

CritCOMPARESTRICT::~CritCOMPARESTRICT() {}

bool CritCOMPARESTRICT::operator()(EntryRef *entry) {
  Attribute attrval = entry->Lookup(attribute);
  if (! attrval.IsString())
    return true;
  return (*ordering)(attrval.StringValue(), value) <= 0;
}

CritEQUAL::~CritEQUAL() {}

bool CritEQUAL::operator()(EntryRef *entry) {
  Attribute attrval = entry->Lookup(attribute);
  if (! attrval.IsValued()) return true;
  if (! attrval.IsValued() && ! value) return false;
  if (! value) return true;
  Attribute::values_t values;
  attrval.MultiValue(values);
  for (Attribute::values_t::const_iterator it = values.begin();
       it != values.end();
       ++it)
    if ((*equality)(*it, *value))
      return false;
  return true;
}

CritNOT::~CritNOT() {}

bool CritNOT::operator()(EntryRef *entry) {
  return ! (*clause)(entry);
}

CritOR::~CritOR() {}

bool CritOR::operator()(EntryRef *entry) {
  return ! ((*first)(entry) || (*second)(entry));
}

CritPREFIX::~CritPREFIX() {}

bool CritPREFIX::operator()(EntryRef *entry) {
  Attribute attrval = entry->Lookup(attribute);
  if (! attrval.IsValued()) return true;
  Attribute::values_t values;
  attrval.MultiValue(values);
  for (Attribute::values_t::const_iterator it = values.begin();
       it != values.end();
       ++it)
    if ((*match)(*it, value, true))
      return false;
  return true;
}


CritRANGE::CritRANGE(unsigned long _low,
		     unsigned long _high,
		     const String &_value)
  : low(_low), high(_high), value(_value) {
    if (low == 0 || high == 0)
      throw Response::BAD("Range requires non-zero parameters");
}

CritRANGE::~CritRANGE() {}

bool CritRANGE::operator()(EntryRef *entry) {
  DEBUG("Checking range of entry " << entry->Name() << "; EntryNumber is " << entry->EntryNumber() << "; low is " << low << "; high is " << high);
  return (entry->EntryNumber() < low
	  || entry->EntryNumber() > high);
}

CritSUBSTRING::~CritSUBSTRING() {}

bool CritSUBSTRING::operator()(EntryRef *entry) {
  Attribute attrval = entry->Lookup(attribute);
  if (! attrval.IsValued()) return true;
  Attribute::values_t values;
  attrval.MultiValue(values);
  for (Attribute::values_t::const_iterator it = values.begin();
       it != values.end();
       ++it)
    if ((*match)(*it, value, false))
      return false;
  return true;
}
