/* dstring.h -*- C++ -*- disk-string interface
 * Rob Earhart
 * $Id: dstring.h,v 1.1 1998/02/18 21:17:46 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_DSTRING_H
#define ACAP_DSTRING_H

#include <iostream.h>
#include "dsobjects.h"
#include "str.h"

class DString {
public:
  typedef String::hash_type hash_type;
  typedef String::size_type size_type;
  typedef String::const_iterator const_iterator;

  DString(hash_type _hash, string_t *_data)
    : hashvalue(_hash), data(_data) {}
  DString(stringval_t *sv)
    : hashvalue(sv->hash), data(sv->str) {}
  DString(const DString &it)
    : hashvalue(it.hashvalue), data(it.data) {}

  operator String() const {
    // This is probably suboptimal in the long run, since we'll
    // want to be able to distribute DStrings over multiple
    // byte ranges.  It works for now, though... XXX
    return String(data->begin(), size(), hash());
  }

  bool operator==(const String &it) const {
    return hash() == it.hash()
        && data->size == it.size()
        && ! lexicographical_compare_3way(begin(), end(),
					  it.begin(), it.end());
  }

  bool operator==(const char *it) const {
    return ! strcmp(begin(), it);
  }

  bool operator!=(const char *it) const {
    return strcmp(begin(), it);
  }

  bool operator==(const DString &it) const {
    return hash() == it.hash()
        && data->size == it.size()
        && ! lexicographical_compare_3way(begin(), end(),
					  it.begin(), it.end());
  }

  bool operator<(const DString &it) const {
    return lexicographical_compare(begin(), end(),
				   it.begin(), it.end());
  }

  hash_type hash() const { return hashvalue; }
  size_type size() const { return data->size; }

  const_iterator begin() const { return data->begin(); }
  const_iterator end() const { return data->end(); }

  friend ostream &operator<<(ostream &s, const DString &d) {
    return (s.write(d.data->begin(), d.size()));
  }

private:
  hash_type hashvalue;
  string_t *data;
};

struct hash<DString> {
  size_t operator()(const DString &str) const {
    return str.hash();
  }
};

#endif // ACAP_DSTRING_H
