/* en-us.cc - en-us language definition
 * Rob Earhart
 * $Id: en-us.cc,v 1.7 1998/03/09 00:58:15 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "msg.h"
#include "language.h"
#include "init.h"

static class En_US_Messages : public Msg {
public:
  // Commands
  const char *Authenticate() { return "Authenticate Complete"; }
  const char *Bye() { return "Cya later, alligator!"; }
  const char *DeletedSince() { return "DeletedSince Complete"; }
  const char *FreeContext() { return "FreeContext Complete"; }
  const char *Lang() { return "Lang Complete"; }
  const char *Noop() { return "NoOp Complete"; }
  const char *Store() { return "Store Complete"; }
  const char *Search() { return "Search Complete"; }
  const char *UpdateContext() { return "UpdateContext Complete"; }

  // Parsing
  const char *CommandAborted() { return "Command aborted"; }
  const char *ExpectedSpace() { return "Expected space"; }
  const char *ExpectedCRLF() { return "Expected the end of the line"; }
  const char *ExpectedLParen() { return "Expected a '('"; }
  const char *ExpectedRParen() { return "Expected a ')'"; }
  const char *ExpectedRParenOrSpace() { return "Expected a ')' or a space"; }
  const char *ExpectedString() { return "Expected a string"; }
  const char *ExpectedStringOrNil() { return "Expected a string or a NIL"; }
  const char *ExpectedQString() { return "Expected a quoted string"; }
  const char *ExpectedNumber() { return "Expected a number"; }
  const char *ExpectedContext() { return "Expected at least one context"; }
  const char *CTypeInEntryAttr() { return "You can't store into the content-type of an entry attribute (it's always text/plain)"; }
  const char *NumberTooBig() { return "Numbers that big are illegal in ACAP"; }
  const char *UnrecWriteMetadata() { return "Unrecognized write-metadata type"; }
  const char *ACLString() { return "ACLs must be identifier lists, not single strings"; }
  const char *CTypeStrList() { return "Content-type metadata must be a simple string, not a string list"; }
  const char *NoMultiInEntry() { return "You can't store a multivalue into an entry attribute."; }
  const char *ExpectedValue() { return "Expected a NIL, DEFAULT, string, or stringlist"; }
  const char *ExpectedStoreMetadata() { return "Expected more store-metadata or a ')'"; }
  const char *NoSuchContext() { return "No such context"; }
  const char *NotUpdateableContext() { return "Not an updateable context"; }
  const char *InvalidSubdatasetSyntax() { return "Invalid Subdataset URL syntax"; }
  const char *MultivaluedInherit() { return "dataset.inherit may not be multivalued"; }

  const char *IllegalPath() { return "Illegal dataset path specified"; }
  const char *UnknownDatamechName() { return "Unknown dataset mechanism name discovered"; }
  const char *DatasetNoExist() { return "Dataset does not exist"; }
  const char *TooOld() { return "Don't have information going back that far"; }
  const char *DatasetReferal() { return "Dataset exists on another server"; }
} en_us_messages;

static Language en_us(&en_us_messages, "en", "en-us");

static void SetupEnUS() {
  Language::Langs()["en"] = &en_us;
  Language::Langs()["en-us"] = &en_us;
}

static Init init(&SetupEnUS);

// This symbol's defined in this file only because en-us is
// our default language, and is therefore "special" in some sense.
const Language *const Language::def = &en_us;
