/* main.cc - acapd entry point
 * Rob Earhart
 * $Id: main.cc,v 1.3 1998/02/18 21:18:39 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <sys/types.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include "acap.h"
#include "unixerror.h"
#include "conn.h"
#include "dispatch.h"

static const char rcsid[] = "$Id: main.cc,v 1.3 1998/02/18 21:18:39 rob Exp $";

typedef struct {
  int conn;
  struct sockaddr_in peer;
} conn_info;

static void *handle_conn(conn_info *info) {
  try {
#ifdef TDEBUG
    int p[2] = {-1, -1};
#endif // TDEBUG
    try {
#ifdef TDEBUG
      // When debugging, we do a cheap hack, and throw both streams
      // to stderr, using our friend Mr. tee...
      if (socketpair(PF_UNIX, SOCK_STREAM, 0, p))
	throw UnixError("socketpair");
      pid_t pid = fork();
      if (pid < 0)
	throw UnixError("fork");
      else if (pid == 0) {
	dup2(info->conn, 0);
	close(info->conn);
	dup2(p[0], 1);
	close(p[0]);
	close(p[1]);
	execl("/bin/tee", "/bin/tee", "/dev/fd/2", NULL);
	_exit(1);
      }
      pid = fork();
      if (pid < 0)
	throw UnixError("fork");
      else if (pid == 0) {
	dup2(p[0], 0);
	close(p[0]);
	dup2(info->conn, 1);
	close(info->conn);
	close(p[1]);
	execl("/bin/tee", "/bin/tee", "/dev/fd/2", NULL);
	_exit(1);
      }
      close(info->conn);
      P<Conn> handler = new Conn(p[1], info->peer);
#else
      P<Conn> handler = new Conn(info->conn, info->peer);
#endif // TDEBUG
      handler->Loop();
    } catch (...) {
      static const char msg[] = "* Bye \"Insufficient resources to establish a connection; sorry!\"\r\n";
      write(info->conn, msg, sizeof(msg) - 1);
      close(info->conn);
#ifdef TDEBUG
      if (0 <= p[0]) close(p[0]);
      if (0 <= p[1]) close(p[1]);
#endif // TDEBUG
    }
    delete info;
  } catch (...) {
    // Paranoia; I *really* don't want to know what'll happen if
    // an exception gets thrown through the top of a pthreads stack...
  }
  return NULL;
}

int main(int argc, char *argv[]) {
  int sock;

  try {
    sock = ACAP::Init(argc, argv);
    DEBUG("Ready for connections");
    while(1) {
      conn_info *info = new conn_info;
      int len = sizeof(info->peer);
      info->conn = accept(sock, (struct sockaddr *)&info->peer, &len);
      pthread_create(NULL, NULL,
		     (void *(*)(void *))&handle_conn, (void *)info);
    }
#if 0
  } catch (Response::StatusResponse m) {
    syslog(LOG_ALERT, "%s: %s\n", argv[0], m.Msg());
    cerr << argv[0] << ": " << m.Msg() << endl;
#endif
  } catch (Error e) {
    syslog(LOG_ALERT, "%s: %s: %s.", argv[0], e.op(), e.err());
    cerr << argv[0] << ": " << e.op() << ": " << e.err() << '.' << endl;
  }
  syslog(LOG_ALERT, "ACAP server exiting.");
  return 1;
}
