/* re.h -*- C++ -*- regexp interface
 * Rob Earhart
 * $Id: re.h,v 1.1 1998/02/18 21:19:13 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_RE_H
#define ACAP_RE_H

// This provides a standard wrapper around whatever system-specific
// regular expression package we wind up using.  We're only really
// depending on lowest-common-denominator regular expression syntax,
// so this is pretty straightforward; it'd be cool, in the future,
// to open up specific RE package capabilities (subpart matching,
// in particular, would be Cool).

#include <sys/types.h>
#include <regex.h>
#include "str.h"
#include "error.h"

class RegExp {
public:
  RegExp(const char *pattern) {
    // We probably ought to throw a more meaningful message... XXX
    int err = regcomp(&re, pattern, REG_EXTENDED | REG_NOSUB);
    if (err) throw Error("Compiling a regexp", pattern);
  }
  ~RegExp() {
    regfree(&re);
  }
  bool Accepts(const char *str) const {
    if (regexec(&re, str, 0, NULL, 0) == 0) return true;
    return false;
  }
  bool Accepts(const String &str) const {
    if (regexec(&re, str.c_str(), 0, NULL, 0) == 0) return true;
    return false;
  }
private:
  regex_t re;
};

#endif // ACAP_RE_H
