/* response.cc - response implementation
 * Rob Earhart
 * $Id: response.cc,v 1.2 1998/02/19 18:06:01 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "response.h"
#include "clientstr.h"
#include "conn.h"
#include "command.h"
#include "language.h"

ostream &operator<<(ostream &o, Response::StatusResponse &c) {
  // We assume that the tag (but not the trailing space!)
  // has been written out...
  return (c.Reason(o << ' ' << c.Opcode() << ' ')
	             << '\"' << c.msg << "\"\r\n" << flush);
}

const char *Response::StatusResponse::GetMessage(Msg::Generator gen) {
    Conn *c = Command::CurrentConn();
    if (c)
      return (c->CurrentLanguage()->Messages()->*gen)();
    else
      return (Language::Default()->Messages()->*gen)();
}

const char *Response::OK::Opcode() const { return "Ok"; }

const char *Response::NO::Opcode() const { return "No"; }

const char *Response::BAD::Opcode() const { return "Bad"; }

ostream &Response::TooOld::Reason(ostream &o) {
  return (o << "(TooOld) ");
}

ostream &Response::NoExist::Reason(ostream &o) {
  return (o << "(NoExist " << ClientStr(path) << ") ");
}

ostream &Response::Invalid::Reason(ostream &o) {
  return (o << "(Invalid " << ClientStr(attribute) << ") ");
}

ostream &Response::Refer::Reason(ostream &o) {
  o << "(Refer";
  for (vector<String>::const_iterator it = urls.begin();
       it != urls.end();
       ++it)
    o << ' ' << ClientStr(*it);
  o << ") ";
  return o;
}
