/* response.h -*- C++ -*- response interface
 * Rob Earhart
 * $Id: response.h,v 1.2 1998/02/19 18:06:03 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_RESPONSE_H
#define ACAP_RESPONSE_H

#include <iostream.h>
#include "msg.h"
#include "str.h"

// Response namespace

class Response {
public:

  class StatusResponse {
  public:
    friend ostream &operator<<(ostream &o, StatusResponse &c);
  protected:
    StatusResponse(const char *_msg) : msg(_msg) {}
    virtual const char *Opcode() const = 0;
    // Reason functions should either do nothing (default), or
    // print an ACAP resp-code (in parens), followed by a space.
    virtual ostream &Reason(ostream &o) { return o; }
    static const char *GetMessage(Msg::Generator gen);
  private:
    const char *msg;
  };

  class OK : public StatusResponse {
  public:
    OK(Msg::Generator gen) : StatusResponse(GetMessage(gen)) {}
  private:
    const char *command;
    const char *Opcode() const;
  };

  class NO : public StatusResponse {
  public:
    NO(const char *_msg) : StatusResponse(_msg) {}
    NO(Msg::Generator gen) : StatusResponse(GetMessage(gen)) {}
  private:
    const char *Opcode() const;
  };

  class BAD : public StatusResponse {
  public:
    BAD(const char *_msg) : StatusResponse(_msg) {}
    BAD(Msg::Generator gen) : StatusResponse(GetMessage(gen)) {}
  private:
    const char *Opcode() const;
  };

  class TooOld : public NO {
  public:
    TooOld() : NO(&Msg::TooOld) {}
  private:
    ostream &Reason(ostream &o);
  };

  class NoExist : public NO {
  public:
    NoExist(const String &_path, Msg::Generator gen)
      : NO(gen), path(_path) {}
  private:
    ostream &Reason(ostream &o);
    String path;
  };

  class Invalid : public BAD {
  public:
    Invalid(const String &_attribute, Msg::Generator gen)
      : BAD(gen), attribute(_attribute) {}
  private:
    ostream &Reason(ostream &o);
    String attribute;
  };

  class Refer : public NO {
  public:
    Refer(const vector<String> &_urls, Msg::Generator gen)
      : NO(gen), urls(_urls) {}
  private:
    ostream &Reason(ostream &o);
    vector<String> urls;
  };
private:
  Response();			// no one gets to construct this...
};

#endif // ACAP_RESPONSE_H
