/* storeentry.h -*- C++ -*- storeentry interface
 * Rob Earhart
 * $Id: storeentry.h,v 1.5 1998/03/09 00:58:53 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_STOREENTRY_H
#define ACAP_STOREENTRY_H

#include <slist.h>
#include <hash_map.h>
#include "token.h"
#include "entry.h"
#include "dataset.h"

class StoreEntry {
public:
  class Value {
  public:
    typedef enum {
      NIL,
      DEFAULT,
      STRING,
      STRINGLIST
    } value_t;
    typedef vector<String> values_t;
    Value(value_t vt) : value_type(vt) {}

    value_t GetValueType() const { return value_type; }
    values_t &GetValues() { return values; }
    const values_t &GetValues() const { return values; }
  private:
    value_t value_type;
    values_t values;
  };

  class Attribute {
  public:
    Attribute(const String &_name, Token::token _special)
      : name(_name), special(_special) {}

    void SetValueNil() {
      Ptr<Value> v = new Value(Value::NIL);
      AddValue(Token::A_VALUE, v);
    }
    void SetValueDefault() {
      Ptr<Value> v = new Value(Value::DEFAULT);
      AddValue(Token::A_VALUE, v);
    }
    void AddValue(Token::token type, Ptr<Value> &_value);
    const String &Name() const { return name; }
    Token::token Special() const { return special; }

    Value *GetACL() const { return acl; }
    Value *GetValue() const { return value; }
    Value *GetCType() const { return ctype; }
  private:
    String name;
    Token::token special;
    Ptr<Value> acl;
    Ptr<Value> value;
    Ptr<Value> ctype;
  };

  typedef hash_map<String, Ptr<Attribute>,
                   hash<String>, equal_to<String> > attributes_t;

  StoreEntry(const String &name);
  void AddAttribute(Ptr<Attribute> &attr);
  void SetUnchangedSince(const String &str);
  void SetNoCreate();

  bool GetNoCreate() const{ return nocreate; }
  const String &Name() const { return name; }
  const String &DatasetName() const { return dataset_name; }
  const String &EntryName() const { return entry_name; }
  bool NoCreate() const { return nocreate; }
  bool GotUnchangedSince() const { return gotunchanged; }
  const String &UnchangedSince() const { return unchangedsince; }
  const attributes_t &Attributes() const { return attributes; }
  const Attribute *EntryAttribute() const { return entry_attr; }
  const Attribute *SubdatasetAttribute() const { return subdataset_attr; }
  const Attribute *InheritAttribute() const { return inherit_attr; }

  void SetDSEntry(Dataset *_dataset, Entry _entry) {
    dataset = _dataset;
    entry = _entry;
  }
  Dataset *GetDataset() const { return dataset; }
  Entry GetEntry() const { return entry; }
private:
  String name;
  String dataset_name;
  String entry_name;
  bool gotunchanged;
  bool nocreate;
  String unchangedsince;
  attributes_t attributes;
  Ptr<Attribute> entry_attr, subdataset_attr;
  Attribute *inherit_attr;

  P<Dataset> dataset;
  Entry entry;
};

#endif // ACAP_STOREENTRY_H
