/* upddatecontext.cc - updatecontext command implementation
 * Rob Earhart
 * $Id: updatecontext.cc,v 1.4 1998/02/18 21:20:23 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "updatecontext.h"

Msg::Generator UpdateContext::Name() const {
  return &Msg::UpdateContext;
}

UpdateContext::UpdateContext(const String &_tag, Conn *_conn,
			     contexts_t &_contexts)
  : Command(_tag, _conn) {
    contexts.swap(_contexts);
    stamp = Interlock::Instance()->CreateStamp();
}

void UpdateContext::Eval() {
  // XXX There's really no reason not to do this in parallel;
  // you'd have to keep the connection from chewing up threads, but
  // honestly, a connection could do that anyway (by issuing multiple
  // UpdateContext commands), so we need some sort of connection-level
  // throttle somewhere anyway.

  contexts.reverse();

  // Okay, first, we make sure that all of the contexts which we found
  // as of the time we processed the command really are valid contexts
  // and can be updated...
  for (contexts_t::iterator it = contexts.begin();
       it != contexts.end();
       ++it) {
    Grab g((*it)->StateMutex());
    if ((*it)->State() != Context::good)
      throw Response::NO(&Msg::NoSuchContext);
  }

  // And then we actually do the updates.
  while(! contexts.empty()) {
    contexts.front()->Update(stamp);
    contexts.pop_front();
  }
}
